@@ -40,6 +40,7 @@ struct connman_device {
 	connman_bool_t offlinemode;
 	connman_bool_t blocked;
 	connman_bool_t powered;
+	connman_bool_t powered_pending;
 	connman_bool_t powered_persistent;
 	connman_bool_t carrier;
 	connman_bool_t scanning;
@@ -86,12 +87,17 @@ static gboolean device_scan_trigger(gpointer user_data)
 	return TRUE;
 }
 
-static void reset_scan_trigger(struct connman_device *device)
+static void clear_scan_trigger(struct connman_device *device)
 {
 	if (device->scan_timeout > 0) {
 		g_source_remove(device->scan_timeout);
 		device->scan_timeout = 0;
 	}
+}
+
+static void reset_scan_trigger(struct connman_device *device)
+{
+	clear_scan_trigger(device);
 
 	if (device->scan_interval > 0) {
 		guint interval = device->scan_interval;
@@ -193,7 +199,7 @@ static int set_connected(struct connman_device *device,
 
 		switch (device->element.ipv4.method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_IGNORE:
 			return 0;
 		case CONNMAN_IPCONFIG_METHOD_STATIC:
 			type = CONNMAN_ELEMENT_TYPE_IPV4;
@@ -249,16 +255,11 @@ static int set_connected(struct connman_device *device,
 
 static int set_carrier(struct connman_device *device, connman_bool_t carrier)
 {
-	struct connman_service *service;
-
 	if (carrier == TRUE)
 		__connman_profile_add_device(device);
 	else
 		__connman_profile_remove_device(device);
 
-	service = __connman_service_lookup_from_device(device);
-	__connman_service_set_carrier(service, carrier);
-
 	return set_connected(device, carrier);
 }
 
@@ -296,7 +297,7 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	DBG("device %p powered %d", device, powered);
 
-	if (device->powered == powered)
+	if (device->powered_pending == powered)
 		return -EALREADY;
 
 	if (!driver)
@@ -306,12 +307,18 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	if (powered == TRUE) {
 		if (driver->enable) {
+			device->powered_pending = powered;
+
 			err = driver->enable(device);
 			if (err == 0)
 				__connman_notifier_enable(type);
 		} else
 			err = -EINVAL;
 	} else {
+		device->powered_pending = powered;
+
+		clear_scan_trigger(device);
+
 		g_hash_table_remove_all(device->networks);
 
 		set_carrier(device, FALSE);
@@ -326,7 +333,9 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	if (err == 0) {
 		device->powered = powered;
-		powered_changed(device);
+
+		if (device->registered == TRUE)
+			powered_changed(device);
 	}
 
 	return err;
@@ -537,6 +546,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
 		int err;
 
+		if (device->ipconfig == NULL)
+			return __connman_error_invalid_property(msg);
+
 		switch (device->mode) {
 		case CONNMAN_DEVICE_MODE_UNKNOWN:
 		case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
@@ -666,6 +678,7 @@ static void unregister_interface(struct connman_element *element)
 
 static int setup_device(struct connman_device *device)
 {
+	enum connman_service_type type;
 	int err;
 
 	DBG("device %p", device);
@@ -678,6 +691,9 @@ static int setup_device(struct connman_device *device)
 		return err;
 	}
 
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_register(type);
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
@@ -721,6 +737,8 @@ static void probe_driver(struct connman_element *element, gpointer user_data)
 
 static void remove_device(struct connman_device *device)
 {
+	enum connman_service_type type;
+
 	DBG("device %p", device);
 
 	__connman_device_disable(device);
@@ -736,6 +754,9 @@ static void remove_device(struct connman_device *device)
 		break;
 	}
 
+	type = __connman_device_get_service_type(device);
+	__connman_notifier_unregister(type);
+
 	unregister_interface(&device->element);
 
 	if (device->driver->remove)
@@ -846,7 +867,10 @@ static void device_destruct(struct connman_element *element)
 	g_free(device->control);
 	g_free(device->interface);
 
-	connman_ipconfig_unref(device->ipconfig);
+	if (device->ipconfig != NULL) {
+		connman_ipconfig_unref(device->ipconfig);
+		device->ipconfig = NULL;
+	}
 
 	g_free(device->last_network);
 
@@ -928,12 +952,6 @@ struct connman_device *connman_device_create(const char *node,
 		break;
 	}
 
-	device->ipconfig = connman_ipconfig_create();
-	if (device->ipconfig == NULL) {
-		connman_device_unref(device);
-		return NULL;
-	}
-
 	device->networks = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, unregister_network);
 
@@ -1013,6 +1031,15 @@ const char *connman_device_get_path(struct connman_device *device)
 void connman_device_set_index(struct connman_device *device, int index)
 {
 	device->element.index = index;
+
+	if (device->ipconfig != NULL) {
+		if (index == connman_ipconfig_get_index(device->ipconfig))
+			return;
+
+		connman_ipconfig_unref(device->ipconfig);
+	}
+
+	device->ipconfig = connman_ipconfig_create(index);
 }
 
 /**
@@ -1168,6 +1195,7 @@ int connman_device_set_powered(struct connman_device *device,
 		return -EALREADY;
 
 	device->powered = powered;
+	device->powered_pending = powered;
 
 	type = __connman_device_get_service_type(device);
 
@@ -1199,9 +1227,6 @@ int __connman_device_set_blocked(struct connman_device *device,
 
 	DBG("device %p blocked %d", device, blocked);
 
-	if (device->blocked == blocked)
-		return -EALREADY;
-
 	device->blocked = blocked;
 
 	if (device->offlinemode == TRUE)
@@ -1265,9 +1290,11 @@ int __connman_device_enable(struct connman_device *device)
 	if (!device->driver || !device->driver->enable)
 		return -EOPNOTSUPP;
 
-	if (device->powered == TRUE)
+	if (device->powered_pending == TRUE)
 		return -EALREADY;
 
+	device->powered_pending = TRUE;
+
 	err = device->driver->enable(device);
 	if (err < 0)
 		return err;
@@ -1304,6 +1331,13 @@ int __connman_device_disable(struct connman_device *device)
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
+	if (device->powered_pending == FALSE)
+		return -EALREADY;
+
+	device->powered_pending = FALSE;
+
+	clear_scan_trigger(device);
+
 	g_hash_table_remove_all(device->networks);
 
 	err = device->driver->disable(device);
@@ -1612,22 +1646,12 @@ int __connman_device_set_offlinemode(connman_bool_t offlinemode)
 
 void __connman_device_increase_connections(struct connman_device *device)
 {
-	enum connman_service_type type;
-
 	device->connections++;
-
-	type = __connman_device_get_service_type(device);
-	__connman_notifier_connect(type);
 }
 
 void __connman_device_decrease_connections(struct connman_device *device)
 {
-	enum connman_service_type type;
-
 	device->connections--;
-
-	type = __connman_device_get_service_type(device);
-	__connman_notifier_disconnect(type);
 }
 
 /**
@@ -1735,14 +1759,9 @@ void __connman_device_set_network(struct connman_device *device,
  */
 int connman_device_register(struct connman_device *device)
 {
-	enum connman_service_type type;
-
 	__connman_storage_load_device(device);
 
-	device->offlinemode = __connman_manager_get_offlinemode();
-
-	type = __connman_device_get_service_type(device);
-	__connman_notifier_register(type);
+	device->offlinemode = __connman_profile_get_offlinemode();
 
 	return connman_element_register(&device->element, NULL);
 }
@@ -1755,13 +1774,8 @@ int connman_device_register(struct connman_device *device)
  */
 void connman_device_unregister(struct connman_device *device)
 {
-	enum connman_service_type type;
-
 	__connman_storage_save_device(device);
 
-	type = __connman_device_get_service_type(device);
-	__connman_notifier_unregister(type);
-
 	connman_element_unregister(&device->element);
 }
 
@@ -1856,6 +1870,7 @@ static struct connman_driver device_driver = {
 
 static int device_load(struct connman_device *device)
 {
+	const char *ident = __connman_profile_active_ident();
 	GKeyFile *keyfile;
 	GError *error = NULL;
 	gchar *identifier;
@@ -1864,7 +1879,7 @@ static int device_load(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open();
+	keyfile = __connman_storage_open(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1895,19 +1910,20 @@ static int device_load(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(keyfile, FALSE);
+	__connman_storage_close(ident, keyfile, FALSE);
 
 	return 0;
 }
 
 static int device_save(struct connman_device *device)
 {
+	const char *ident = __connman_profile_active_ident();
 	GKeyFile *keyfile;
 	gchar *identifier;
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open();
+	keyfile = __connman_storage_open(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1933,7 +1949,7 @@ static int device_save(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(keyfile, TRUE);
+	__connman_storage_close(ident, keyfile, TRUE);
 
 	return 0;
 }
