@@ -756,6 +756,12 @@ static DBusMessage *connect_service(DBusConnection *conn,
 
 	err = __connman_service_connect(service);
 	if (err < 0) {
+		if (err == -ENOKEY) {
+			if (__connman_agent_request_passphrase(service,
+							NULL, NULL) == 0)
+				return NULL;
+		}
+
 		if (err != -EINPROGRESS) {
 			dbus_message_unref(service->pending);
 			service->pending = NULL;
@@ -1796,6 +1802,11 @@ struct connman_service *__connman_service_lookup_from_network(struct connman_net
 	return service;
 }
 
+const char *__connman_service_get_path(struct connman_service *service)
+{
+	return service->path;
+}
+
 unsigned int __connman_service_get_order(struct connman_service *service)
 {
 	GSequenceIter *iter;
@@ -2022,9 +2033,11 @@ static int service_load(struct connman_service *service)
 {
 	const char *ident = service->profile;
 	GKeyFile *keyfile;
+	GError *error = NULL;
 	gchar *pathname, *data = NULL;
 	gsize length;
 	gchar *str;
+	connman_bool_t autoconnect;
 	unsigned int ssid_len;
 	int err = 0;
 
@@ -2114,8 +2127,11 @@ static int service_load(struct connman_service *service)
 		service->favorite = g_key_file_get_boolean(keyfile,
 				service->identifier, "Favorite", NULL);
 
-		service->autoconnect = g_key_file_get_boolean(keyfile,
-				service->identifier, "AutoConnect", NULL);
+		autoconnect = g_key_file_get_boolean(keyfile,
+				service->identifier, "AutoConnect", &error);
+		if (error == NULL)
+			service->autoconnect = autoconnect;
+		g_clear_error(&error);
 
 		str = g_key_file_get_string(keyfile,
 				service->identifier, "Failure", NULL);
